# NGram documentation build configuration file.

# Add any Sphinx extension module names here, as strings. They can be extensions
# coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = ['sphinx.ext.autodoc', 'sphinx.ext.doctest']

# The suffix of source filenames.
source_suffix = '.rst'

# The human-readable project name
project = u'Python NGram'

# The short X.Y version.
version = '4.0'

# The full version, including alpha/beta/rc tags.
release = '4.0.3'

# Language the docs are written in.
language = 'en'

# Copyright byline.
copyright = 'Graham Poulter, Michel Albert 2008-2021'

# List of directories, relative to source directory, that shouldn't be searched
# for source files.
exclude_trees = ['build']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# Name (excluding extension) of root document.
root_doc = 'index'


"""
:mod:`ngram` -- A set class that supports lookup by N-gram string similarity
============================================================================
"""

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

import warnings

class NGram(set):
    """A set that supports searching for members by N-gram string similarity.

    In Python 2, items should be `unicode` string or a plain ASCII `str`
    (bytestring) - do not use UTF-8 or other multi-byte encodings, because
    multi-byte characters will be split up.

    :type threshold: float in 0.0 ... 1.0

    :param threshold: minimum similarity for a string to be considered a match.

    :type warp: float in 1.0 ... 3.0

    :param warp: use warp greater than 1.0 to increase the similarity of \
    shorter string pairs.

    :type items: [item, ...]

    :param items: iteration of items to index for N-gram search.

    :type N: int >= 2

    :param N: number of characters per n-gram.

    :type pad_len: int in 0 ... N-1

    :param pad_len: how many characters padding to add (defaults to N-1).

    :type pad_char: str or unicode

    :param pad_char: character to use for padding.  Default is '$', but \
    consider using the\\ non-breaking space character, ``u'\\xa0'`` \
    (``u"\\u00A0"``).

    :type key: function(item) -> str/unicode

    :param key: Function to convert items into string, default is no \
    conversion.  Recommended to use `str` or `unicode` for non-string items. \
    Using anonymous function prevents NGram class from being pickled.

    Instance variables:

    :ivar _grams: For each n-gram, the items containing it and the number of \
    times\\ the n-gram occurs in the item as ``{str:{item:int, ...}, ...}``.

    :ivar length: maps items to length of the padded string representations \
    as ``{item:int, ...}``.
    """

    def __init__(self, items=None, threshold=0.0, warp=1.0, key=None,
                    N=3, pad_len=None, pad_char='$', **kwargs):
        super(NGram, self).__init__()
        if not (0 <= threshold <= 1):
            raise ValueError("threshold out of range 0.0 to 1.0: "
                             + repr(threshold))
        if not (1.0 <= warp <= 3.0):
            raise ValueError(
                "warp out of range 1.0 to 3.0: " + repr(warp))
        if not N >= 1:
            raise ValueError("N out of range (should be N >= 1): " + repr(N))
        if pad_len is None:
            pad_len = N - 1
        if not (0 <= pad_len < N):
            raise ValueError("pad_len out of range: " + repr(pad_len))
        if not len(pad_char) == 1:
            raise ValueError(
                "pad_char is not single character: " + repr(pad_char))
        if key is not None and not callable(key):
            raise ValueError("key is not a function: " + repr(key))
        self.threshold = threshold
        self.warp = warp
        self.N = N
        self._pad_len = pad_len
        self._pad_char = pad_char
        self._padding = pad_char * pad_len  # derive a padding string
        # compatibility shim for 3.1 iconv parameter
        if 'iconv' in kwargs:
            self._key = kwargs.pop('iconv')
            warnings.warn('"iconv" parameter deprecated, use "key" instead.', DeprecationWarning)
        # no longer support 3.1 qconv parameter
        if 'qconv' in kwargs:
            raise ValueError('qconv query conversion parameter unsupported. '
            'Please process query to a string before calling .search')
        self._key = key
        self._grams = {}
        self.length = {}
        if items:
            self.update(items)

    def __reduce__(self):
        """Return state information for pickling, no references to this
        instance.  The key function must be None, a builtin function, or
        a named module-level function.

        >>> from ngram import NGram
        >>> n = NGram([0xDEAD, 0xBEEF], key=hex)
        >>> import pickle
        >>> p = pickle.dumps(n)
        >>> m = pickle.loads(p)
        >>> sorted(list(m))
        [48879, 57005]
        """
        return NGram, (list(self), self.threshold, self.warp, self._key,
                       self.N, self._pad_len, self._pad_char)

    def copy(self, items=None):
        """Return a new NGram object with the same settings, and
        referencing the same items.  Copy is shallow in that
        each item is not recursively copied.   Optionally specify
        alternate items to populate the copy.

        >>> from ngram import NGram
        >>> from copy import deepcopy
        >>> n = NGram(['eggs', 'spam'])
        >>> m = n.copy()
        >>> m.add('ham')
        >>> sorted(list(n))
        ['eggs', 'spam']
        >>> sorted(list(m))
        ['eggs', 'ham', 'spam']
        >>> p = n.copy(['foo', 'bar'])
        >>> sorted(list(p))
        ['bar', 'foo']
        """
        return NGram(items if items is not None else self,
                     self.threshold, self.warp, self._key,
                     self.N, self._pad_len, self._pad_char)

    def key(self, item):
        """Get the key string for the item.

        >>> from ngram import NGram
        >>> n = NGram(key=lambda x:x[1])
        >>> n.key((3,"ham"))
        'ham'
        """
        return self._key(item) if self._key else item

    def pad(self, string):
        """Pad a string in preparation for splitting into ngrams.

        >>> from ngram import NGram
        >>> n = NGram()
        >>> n.pad('ham')
        '$$ham$$'
        """
        return self._padding + string + self._padding

    def _split(self, string):
        """Iterates over the ngrams of a string (no padding).

        >>> from ngram import NGram
        >>> n = NGram()
        >>> list(n._split("hamegg"))
        ['ham', 'ame', 'meg', 'egg']
        """
        for i in range(len(string) - self.N + 1):
            yield string[i:i + self.N]

    def split(self, string):
        """Pads a string and iterates over its ngrams.

        >>> from ngram import NGram
        >>> n = NGram()
        >>> list(n.split("ham"))
        ['$$h', '$ha', 'ham', 'am$', 'm$$']
        """
        return self._split(self.pad(string))

    def ngrams(self, string):
        """Alias for 3.1 compatibility, please set pad_len=0 and use split.""" 
        warnings.warn('Method ngram deprecated, use method split with pad_len=0 instead.', DeprecationWarning)
        return self._split(string)

    def ngrams_pad(self, string):
        """Alias for 3.1 compatibility, please use split instead."""
        warnings.warn('Method ngrams_pad deprecated, use method split instead.', DeprecationWarning)
        return self.split(string)

    def splititem(self, item):
        """Pads the string key of an item and iterates over its ngrams.

        >>> from ngram import NGram
        >>> n = NGram(key=lambda x:x[1])
        >>> item = (3,"ham")
        >>> list(n.splititem(item))
        ['$$h', '$ha', 'ham', 'am$', 'm$$']
        """
        return self.split(self.key(item))

    def add(self, item):
        """Add an item to the N-gram index (if it has not already been added).

        >>> from ngram import NGram
        >>> n = NGram()
        >>> n.add("ham")
        >>> list(n)
        ['ham']
        >>> n.add("spam")
        >>> sorted(list(n))
        ['ham', 'spam']
        """
        if item not in self:
            # Add the item to the base set
            super(NGram, self).add(item)
            # Record length of padded string
            padded_item = self.pad(self.key(item))
            self.length[item] = len(padded_item)
            for ngram in self._split(padded_item):
                # Add a new n-gram and string to index if necessary
                self._grams.setdefault(ngram, {}).setdefault(item, 0)
                # Increment number of times the n-gram appears in the string
                self._grams[ngram][item] += 1

    def remove(self, item):
        """Remove an item from the set. Inverts the add operation.

        >>> from ngram import NGram
        >>> n = NGram(['spam', 'eggs'])
        >>> n.remove('spam')
        >>> list(n)
        ['eggs']
        """
        if item in self:
            super(NGram, self).remove(item)
            del self.length[item]
            for ngram in set(self.splititem(item)):
                del self._grams[ngram][item]

    def pop(self):
        """Remove and return an arbitrary set element.
        Raises KeyError if the set is empty.

        >>> from ngram import NGram
        >>> n = NGram(['spam', 'eggs'])
        >>> x = n.pop()
        >>> len(n)
        1
        """
        item = super(NGram, self).pop()
        del self.length[item]
        for ngram in set(self.splititem(item)):
            del self._grams[ngram][item]
        return item

    def items_sharing_ngrams(self, query):
        """Retrieve the subset of items that share n-grams the query string.

        :param query: look up items that share N-grams with this string.
        :return: mapping from matched string to the number of shared N-grams.

        >>> from ngram import NGram
        >>> n = NGram(["ham","spam","eggs"])
        >>> sorted(n.items_sharing_ngrams("mam").items())
        [('ham', 2), ('spam', 2)]
        """
        # From matched string to number of N-grams shared with query string
        shared = {}
        # Dictionary mapping n-gram to string to number of occurrences of that
        # ngram in the string that remain to be matched.
        remaining = {}
        for ngram in self.split(query):
            try:
                for match, count in self._grams[ngram].items():
                    remaining.setdefault(ngram, {}).setdefault(match, count)
                    # match as many occurrences as exist in matched string
                    if remaining[ngram][match] > 0:
                        remaining[ngram][match] -= 1
                        shared.setdefault(match, 0)
                        shared[match] += 1
            except KeyError:
                pass
        return shared

    def searchitem(self, item, threshold=None):
        """Search the index for items whose key exceeds the threshold
        similarity to the key of the given item.

        :return: list of pairs of (item, similarity) by decreasing similarity.

        >>> from ngram import NGram
        >>> n = NGram([(0, "SPAM"), (1, "SPAN"), (2, "EG"),
        ... (3, "SPANN")], key=lambda x:x[1])
        >>> sorted(n.searchitem((2, "SPA"), 0.35))
        [((0, 'SPAM'), 0.375), ((1, 'SPAN'), 0.375)]
        """
        return self.search(self.key(item), threshold)

    def search(self, query, threshold=None):
        """Search the index for items whose key exceeds threshold
        similarity to the query string.

        :param query: returned items will have at least `threshold` \
        similarity to the query string.

        :return: list of pairs of (item, similarity) by decreasing similarity.

        >>> from ngram import NGram
        >>> n = NGram([(0, "SPAM"), (1, "SPAN"), (2, "EG")], key=lambda x:x[1])
        >>> sorted(n.search("SPA"))
        [((0, 'SPAM'), 0.375), ((1, 'SPAN'), 0.375)]
        >>> n.search("M")
        [((0, 'SPAM'), 0.125)]
        >>> n.search("EG")
        [((2, 'EG'), 1.0)]
        """
        threshold = threshold if threshold is not None else self.threshold
        results = []
        # Identify possible results
        for match, samegrams in self.items_sharing_ngrams(query).items():
            allgrams = (len(self.pad(query))
                        + self.length[match] - (2 * self.N) - samegrams + 2)
            similarity = self.ngram_similarity(samegrams, allgrams, self.warp)
            if similarity >= threshold:
                results.append((match, similarity))
        # Sort results by decreasing similarity
        results.sort(key=lambda x: x[1], reverse=True)
        return results

    def finditem(self, item, threshold=None):
        """Return most similar item to the provided one, or None if
        nothing exceeds the threshold.

        >>> from ngram import NGram
        >>> n = NGram([(0, "Spam"), (1, "Ham"), (2, "Eggsy"), (3, "Egggsy")],
        ...     key=lambda x:x[1].lower())
        >>> n.finditem((3, 'Hom'))
        (1, 'Ham')
        >>> n.finditem((4, "Oggsy"))
        (2, 'Eggsy')
        >>> n.finditem((4, "Oggsy"), 0.8)
        """
        results = self.searchitem(item, threshold)
        if results:
            return results[0][0]
        else:
            return None

    def find(self, query, threshold=None):
        """Simply return the best match to the query, None on no match.

        >>> from ngram import NGram
        >>> n = NGram(["Spam","Eggs","Ham"], key=lambda x:x.lower(), N=1)
        >>> n.find('Hom')
        'Ham'
        >>> n.find("Spom")
        'Spam'
        >>> n.find("Spom", 0.8)
        """
        results = self.search(query, threshold)
        if results:
            return results[0][0]
        else:
            return None

    @staticmethod
    def ngram_similarity(samegrams, allgrams, warp=1.0):
        """Similarity for two sets of n-grams.

        :note: ``similarity = (a**e - d**e)/a**e`` where `a` is \
        "all n-grams", `d` is "different n-grams" and `e` is the warp.

        :param samegrams: number of n-grams shared by the two strings.

        :param allgrams: total of the distinct n-grams across the two strings.
        :return: similarity in the range 0.0 to 1.0.

        >>> from ngram import NGram
        >>> NGram.ngram_similarity(5, 10)
        0.5
        >>> NGram.ngram_similarity(5, 10, warp=2)
        0.75
        >>> NGram.ngram_similarity(5, 10, warp=3)
        0.875
        >>> NGram.ngram_similarity(2, 4, warp=2)
        0.75
        >>> NGram.ngram_similarity(3, 4)
        0.75
        """
        if abs(warp - 1.0) < 1e-9:
            similarity = float(samegrams) / allgrams
        else:
            diffgrams = float(allgrams - samegrams)
            similarity = ((allgrams ** warp - diffgrams ** warp)
                    / (allgrams ** warp))
        return similarity

    @staticmethod
    def compare(s1, s2, **kwargs):
        """Compares two strings and returns their similarity.

        :param s1: first string
        :param s2: second string
        :param kwargs: additional keyword arguments passed to __init__.
        :return: similarity between 0.0 and 1.0.

        >>> from ngram import NGram
        >>> NGram.compare('spa', 'spam')
        0.375
        >>> NGram.compare('ham', 'bam')
        0.25
        >>> NGram.compare('spam', 'pam') #N=2
        0.375
        >>> NGram.compare('ham', 'ams', N=1)
        0.5
        """
        if s1 is None or s2 is None:
            if s1 == s2:
                return 1.0
            return 0.0
        try:
            return NGram([s1], **kwargs).search(s2)[0][1]
        except IndexError:
            return 0.0

    ### Set operations implemented on top of NGram add/remove

    def update(self, items):
        """Update the set with new items.

        >>> from ngram import NGram
        >>> n = NGram(["spam"])
        >>> n.update(["eggs"])
        >>> sorted(list(n))
        ['eggs', 'spam']
        """
        for item in items:
            self.add(item)

    def discard(self, item):
        """Remove an element from a set if it is a member.

        If the element is not a member, do nothing.

        >>> from ngram import NGram
        >>> n = NGram(['spam', 'eggs'])
        >>> n.discard('spam')
        >>> n.discard('ham')
        >>> list(n)
        ['eggs']
        """
        if item in self:
            self.remove(item)

    def clear(self):
        """Remove all elements from this set.

        >>> from ngram import NGram
        >>> n = NGram(['spam', 'eggs'])
        >>> sorted(list(n))
        ['eggs', 'spam']
        >>> n.clear()
        >>> list(n)
        []
        """
        super(NGram, self).clear()
        self._grams = {}
        self.length = {}

    def union(self, *others):
        """Return the union of two or more sets as a new set.

        >>> from ngram import NGram
        >>> a = NGram(['spam', 'eggs'])
        >>> b = NGram(['spam', 'ham'])
        >>> sorted(list(a.union(b)))
        ['eggs', 'ham', 'spam']
        """
        return self.copy(super(NGram, self).union(*others))

    def difference(self, *others):
        """Return the difference of two or more sets as a new set.

        >>> from ngram import NGram
        >>> a = NGram(['spam', 'eggs'])
        >>> b = NGram(['spam', 'ham'])
        >>> list(a.difference(b))
        ['eggs']
        """
        return self.copy(super(NGram, self).difference(*others))

    def difference_update(self, other):
        """Remove from this set all elements from `other` set.

        >>> from ngram import NGram
        >>> n = NGram(['spam', 'eggs'])
        >>> other = set(['spam'])
        >>> n.difference_update(other)
        >>> list(n)
        ['eggs']
        """
        for item in other:
            self.discard(item)

    def intersection(self, *others):
        """Return the intersection of two or more sets as a new set.

        >>> from ngram import NGram
        >>> a = NGram(['spam', 'eggs'])
        >>> b = NGram(['spam', 'ham'])
        >>> list(a.intersection(b))
        ['spam']
        """
        return self.copy(super(NGram, self).intersection(*others))

    def intersection_update(self, *others):
        """Update the set with the intersection of itself and other sets.

        >>> from ngram import NGram
        >>> n = NGram(['spam', 'eggs'])
        >>> other = set(['spam', 'ham'])
        >>> n.intersection_update(other)
        >>> list(n)
        ['spam']
        """
        self.difference_update(super(NGram, self).difference(*others))

    def symmetric_difference(self, other):
        """Return the symmetric difference of two sets as a new set.

        >>> from ngram import NGram
        >>> a = NGram(['spam', 'eggs'])
        >>> b = NGram(['spam', 'ham'])
        >>> sorted(list(a.symmetric_difference(b)))
        ['eggs', 'ham']
        """
        return self.copy(super(NGram, self).symmetric_difference(other))

    def symmetric_difference_update(self, other):
        """Update the set with the symmetric difference of itself and `other`.

        >>> from ngram import NGram
        >>> n = NGram(['spam', 'eggs'])
        >>> other = set(['spam', 'ham'])
        >>> n.symmetric_difference_update(other)
        >>> sorted(list(n))
        ['eggs', 'ham']
        """
        intersection = super(NGram, self).intersection(other)
        self.update(other)  # add items present in other
        self.difference_update(intersection)  # remove items present in both



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Unit tests for the ngram module."""

import unittest
from ngram import NGram


class NgramTests(unittest.TestCase):
    """Tests of the ngram class"""

    items = ['sdafaf', 'asfwef', 'asdfawe', 'adfwe', 'askfjwehiuasdfji']

    def test_ngram_search(self):
        """Tests from the original ngram.py, to check that the
        rewrite still uses the same underlying algorithm"""

        # Basic searching of the index
        idx = NGram(self.items)
        self.assertEqual(idx.search('askfjwehiuasdfji'), [
            ('askfjwehiuasdfji', 1.0),
            ('asdfawe', 0.17391304347826086),
            ('asfwef', 0.083333333333333329),
            ('adfwe', 0.041666666666666664)])
        self.assertEqual(idx.search('afadfwe')[:2],
                [('adfwe', 0.59999999999999998),
                 ('asdfawe', 0.20000000000000001)])

        # Pairwise comparison of strings
        self.assertEqual(NGram.compare('sdfeff', 'sdfeff'), 1.0)
        self.assertEqual(NGram.compare('sdfeff', 'zzzzzz'), 0.0)

    def test_set_operations(self):
        """Test advanced set operations"""
        items1 = set(["abcde", "cdefg", "fghijk", "ijklm"])
        items2 = set(["cdefg", "lmnop"])
        idx1 = NGram(items1)
        idx2 = NGram(items2)
        results = lambda L: sorted(x[0] for x in L)
        # Item removal
        self.assertEqual(results(idx1.search('cde')), ["abcde", "cdefg"])
        idx1.remove('abcde')
        self.assertEqual(results(idx1.search('cde')), ["cdefg"])
        # Set intersection operation
        items1.remove('abcde')
        idx1.intersection_update(idx2)
        self.assertEqual(idx1, items1.intersection(items2))
        self.assertEqual(results(idx1.search('lmn')), [])
        self.assertEqual(results(idx1.search('ijk')), [])
        self.assertEqual(results(idx1.search('def')), ['cdefg'])


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python

import os
import os.path
import subprocess
import sys
import tempfile
import textwrap
import unittest

class CsvjoinTests(unittest.TestCase):

    def setUp(self):
        self.tmpdir = tempfile.mkdtemp(prefix="csvjoin_test_")
        self.leftpath = os.path.join(self.tmpdir, 'left')
        self.rightpath = os.path.join(self.tmpdir, 'right')
        self.outpath = os.path.join(self.tmpdir, 'output')
        with open(self.leftpath, 'w') as left:
            left.write('''ID,NAME\n1,Joe\n2,Kin\n3,ZAS''')
        with open(self.rightpath, 'w') as right:
            right.write('''ID,NAME\nID,NAME\nA,Joe\nB,Jon\nC,Job\nD,Kim''')

    def tearDown(self):
        os.remove(self.leftpath)
        os.remove(self.rightpath)
        os.remove(self.outpath)
        os.rmdir(self.tmpdir) 

    def test_csvjoin(self):
        args = [
            sys.executable,
            'scripts/csvjoin.py', '--titles', '-j', 'outer', '--minscore=0.24',
            '--count=5', '--warp=1.0',
            self.leftpath, '1', self.rightpath, '1', self.outpath
        ]
        print(args)
        subprocess.call(args)
        with open(self.outpath, 'r') as out:
            result = '\n'.join(s.strip() for s in out.readlines())
            correct = textwrap.dedent("""\
                ID,NAME,Rank,Similarity,ID,NAME
                1,Joe,1,1.0,A,Joe
                1,Joe,2,0.25,B,Jon
                1,Joe,3,0.25,C,Job
                2,Kin,1,0.25,D,Kim
                3,ZAS""")
            self.assertEqual(result, correct)

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python
"""
Left similarity join between two CSV files.

For each row in the first file, take the specified join column and find
similar rows in the second file based on ngram similarity to a specified
column in the second file. For each resulting pair of rows, output a row
consisting of the fields from the first file, a column with the similarity
value, and then the fields from the second file.
"""
from __future__ import print_function

import csv, os, re
from ngram import NGram

def lowstrip(term):
    """Convert to lowercase and strip spaces"""
    term = re.sub(r'\s+', ' ', term)
    term = term.lower()
    return term

def main(left_path, left_column, right_path, right_column,
         outfile, titles, join, minscore, count, warp):
    """Perform the similarity join"""
    right_file = csv.reader(open(right_path, 'r'))
    if titles:
        right_header = next(right_file)
    index = NGram((tuple(r) for r in right_file),
                  threshold=minscore,
                  warp=warp, key=lambda x: lowstrip(x[right_column]))
    left_file = csv.reader(open(left_path, 'r'))
    out = csv.writer(open(outfile, 'w'), lineterminator='\n')
    if titles:
        left_header = next(left_file)
        out.writerow(left_header + ["Rank", "Similarity"] + right_header)
    for row in left_file:
        if not row: continue # skip blank lines
        row = tuple(row)
        results = index.search(lowstrip(row[left_column]), threshold=minscore)
        if results:
            if count > 0:
                results = results[:count]
            for rank, result in enumerate(results, 1):
                out.writerow(row + (rank, result[1]) + result[0])
        elif join == "outer":
            out.writerow(row)

def console_main():
    """Process command-line arguments."""
    from argparse import ArgumentParser
    parser = ArgumentParser(description=__doc__)
    parser.add_argument('-t', '--titles', action='store_true',
                        help='input files have column titles')
    parser.add_argument(
        '-j', '--join', choices=['inner', 'outer'],
        help=('The kind of left join to perform.  Outer join outputs left-hand '
              'rows which have no right hand match, while inner join discards '
              'such rows. Default: %(default)s'))
    parser.add_argument('-m', '--minscore', type=float,
                        help='Minimum match score: %(default)s')
    parser.add_argument('-c', '--count', type=int,
                help='Max number of rows to match (0 for all): %(default)s')
    parser.add_argument('-w', '--warp', type=float,
            help='N-gram warp, higher helps short strings: %(default)s')
    parser.add_argument('left', nargs=1, help='First CSV file')
    parser.add_argument('leftcolumn', nargs=1, type=int, help='Column in first CSV file')
    parser.add_argument('right', nargs=1, help='Second CSV file')
    parser.add_argument('rightcolumn', nargs=1, type=int, help='Column in second CSV file')
    parser.add_argument('outfile', nargs=1, help='Output CSV file')
    parser.set_defaults(
        titles=False, join='outer', minscore=0.24, count=0, warp=1.0)
    args = parser.parse_args()
    for path in [args.left[0], args.right[0]]:
        if not os.path.isfile(path):
            parser.error('File "%s" does not exist.' % path)
    if not (0 <= args.minscore <= 1.0):
        parser.error("Minimum score must be between 0 and 1")
    if not args.count >= 0:
        parser.error("Maximum number of matches per row must be non-negative.")
    if args.count == 0:
        args.count = None # to return all results
    main(args.left[0], args.leftcolumn[0], args.right[0], args.rightcolumn[0],
         args.outfile[0], args.titles, args.join, args.minscore, args.count,
         args.warp)


if __name__ == '__main__':
    console_main()
